@@ -24,17 +24,20 @@ static engines::engine::BaseOptions _setOptions()
 {
 	engines::engine::BaseOptions s ;
 
-	s.autoMountsOnCreate  = true ;
-	s.hasGUICreateOptions = true ;
-	s.setsCipherPath      = true ;
 	s.supportsMountPathsOnWindows = false ;
-
-	s.configFileArgument   = "--config" ;
-
-	s.configFileNames = QStringList{ ".ecryptfs.config","ecryptfs.config" } ;
-
-	s.fuseNames = QStringList{ "ecryptfs" } ;
-	s.names     = QStringList{ "ecryptfs" } ;
+	s.customBackend         = false ;
+	s.requiresAPassword     = true ;
+	s.hasConfigFile         = true ;
+	s.autoMountsOnCreate    = true ;
+	s.hasGUICreateOptions   = true ;
+	s.setsCipherPath        = true ;
+	s.passwordFormat        = "%{password}" ;
+	s.executableName        = "ecryptfs-simple" ;
+	s.incorrectPasswordText = "error: mount failed" ;
+	s.configFileArgument    = "--config" ;
+	s.configFileNames       = QStringList{ ".ecryptfs.config","ecryptfs.config" } ;
+	s.fuseNames             = QStringList{ "ecryptfs" } ;
+	s.names                 = QStringList{ "ecryptfs" } ;
 
 	s.notFoundCode = engines::engine::status::ecryptfs_simpleNotFound ;
 
@@ -45,8 +48,11 @@ ecryptfs::ecryptfs() : engines::engine( _setOptions() )
 {
 }
 
-engines::engine::args ecryptfs::command( const engines::engine::cmdArgsList& args ) const
+engines::engine::args ecryptfs::command( const QString& password,
+					 const engines::engine::cmdArgsList& args ) const
 {
+	Q_UNUSED( password )
+
 	auto e = QString( "%1 %2 -a %3 %4 %5" ) ;
 
 	engines::engine::commandOptions m( args,QString() ) ;
@@ -89,22 +95,15 @@ engines::engine::args ecryptfs::command( const engines::engine::cmdArgsList& arg
 	}
 }
 
-engines::engine::error ecryptfs::errorCode( const QString& e ) const
-{
-	Q_UNUSED( e ) ;
-
-	return engines::engine::error::Failed ;
-}
-
 engines::engine::status ecryptfs::errorCode( const QString& e,int s ) const
 {
-	Q_UNUSED( s ) ;
+	Q_UNUSED( s )
 
 	if( e.contains( "Operation not permitted" ) ){
 
 		return engines::engine::status::ecrypfsBadExePermissions ;
 
-	}else if( e.contains( "error: mount failed" ) ){
+	}else if( e.contains( this->incorrectPasswordText() ) ){
 
 		return engines::engine::status::ecryptfsBadPassword ;
 	}else{
@@ -112,11 +111,6 @@ engines::engine::status ecryptfs::errorCode( const QString& e,int s ) const
 	}
 }
 
-QString ecryptfs::setPassword( const QString& e ) const
-{
-	return e ;
-}
-
 QString ecryptfs::installedVersionString() const
 {
 	return this->baseInstalledVersionString( "--version",true,1,0 ) ;
