@@ -26,11 +26,9 @@ public:
 
 	engines::engine::status errorCode( const QString& e,int s ) const override ;
 
-	engines::engine::args command( const engines::engine::cmdArgsList& args ) const override ;
+	engines::engine::args command( const QString& password,
+				       const engines::engine::cmdArgsList& args ) const override ;
 
-	engines::engine::error errorCode( const QString& ) const override ;
-
-	QString setPassword( const QString& ) const override ;
 	QString installedVersionString() const override ;
 
 	void GUICreateOptionsinstance( QWidget * parent,engines::engine::function ) const override ;
