@@ -46,6 +46,7 @@
 #include <QTranslator>
 
 #include <QCoreApplication>
+#include <QStandardPaths>
 
 settings::settings() : m_settings( "SiriKali","SiriKali" )
 {
@@ -125,26 +126,6 @@ int settings::sshfsBackendTimeout()
 	return m_settings.value( "sshfsBackendTimeout" ).toInt() ;
 }
 
-void settings::replaceFavorite( const favorites::entry& e,const favorites::entry& f )
-{
-	QStringList l ;
-
-	for( const auto& it : this->readFavorites() ){
-
-		if( it == e ){
-
-			l.append( f.configString() ) ;
-		}else{
-			l.append( it.configString() ) ;
-		}
-	}
-
-	if( !l.isEmpty() ){
-
-		m_settings.setValue( "FavoritesVolumes",l ) ;
-	}
-}
-
 int settings::favoritesEntrySize()
 {
 	auto s = m_settings.value( "FavoritesVolumes" ).toStringList() ;
@@ -214,6 +195,36 @@ bool settings::enableRevealingPasswords()
 	}
 }
 
+bool settings::enableHighDpiScaling()
+{
+	if( !m_settings.contains( "EnableHighDpiScaling" ) ){
+
+		m_settings.setValue( "EnableHighDpiScaling",false ) ;
+	}
+
+	return m_settings.value( "EnableHighDpiScaling" ).toBool() ;
+}
+
+void settings::enableHighDpiScaling( bool e )
+{
+	m_settings.setValue( "EnableHighDpiScaling",e ) ;
+}
+
+QByteArray settings::enabledHighDpiScalingFactor()
+{
+	if( !m_settings.contains( "EnabledHighDpiScalingFactor" ) ){
+
+		m_settings.setValue( "EnabledHighDpiScalingFactor","1.1" ) ;
+	}
+
+	return m_settings.value( "EnabledHighDpiScalingFactor" ).toByteArray() ;
+}
+
+void settings::enabledHighDpiScalingFactor( const QString& e )
+{
+	m_settings.setValue( "EnabledHighDpiScalingFactor",e ) ;
+}
+
 void settings::clearFavorites()
 {
 	m_settings.setValue( "FavoritesVolumes",QStringList() ) ;
@@ -271,35 +282,12 @@ void settings::scaleGUI()
 {
 #if QT_VERSION >= 0x050600
 
-	bool e = [ this ](){
-
-		if( m_settings.contains( "EnableHighDpiScaling" ) ){
-
-			return m_settings.value( "EnableHighDpiScaling" ).toBool() ;
-		}else{
-			bool s = false ;
-			m_settings.setValue( "EnableHighDpiScaling",s ) ;
-			m_settings.setValue( "EnabledHighDpiScalingFactor",QString( "1" ) ) ;
-			return s ;
-		}
-	}() ;
-
-	if( e ){
+	if( this->enableHighDpiScaling() ){
 
 		QApplication::setAttribute( Qt::AA_EnableHighDpiScaling ) ;
 
-		qputenv( "QT_SCALE_FACTOR",[ this ](){
-
-			if( m_settings.contains( "EnabledHighDpiScalingFactor" ) ){
-
-				return m_settings.value( "EnabledHighDpiScalingFactor" ).toString().toLatin1() ;
-			}else{
-				m_settings.setValue( "EnabledHighDpiScalingFactor",QString( "1" ) ) ;
-				return QByteArray( "1" ) ;
-			}
-		}() ) ;
+		qputenv( "QT_SCALE_FACTOR",this->enabledHighDpiScalingFactor() ) ;
 	}
-
 #endif
 }
 
@@ -357,6 +345,38 @@ QString settings::mountPath( const QString& path )
 	return m_settings.value( "MountPrefix" ).toString() + "/" + path ;
 }
 
+QString settings::ConfigLocation()
+{
+	if( !m_settings.contains( "ConfigLocation" ) ){
+
+		auto m = QStandardPaths::standardLocations( QStandardPaths::ConfigLocation ) ;
+
+		if( !m.isEmpty() ){
+
+			m_settings.setValue( "ConfigLocation",m.first() + "/SiriKali/" ) ;
+		}else{
+			//TODO: what to do here???
+		}
+	}
+
+	return m_settings.value( "ConfigLocation" ).toString() ;
+}
+
+QString settings::environmentalVariableVolumeKey()
+{
+	if( !m_settings.contains( "EnvironmentalVariableVolumeKey" ) ){
+
+		m_settings.setValue( "EnvironmentalVariableVolumeKey","SiriKaliVolumeKey" ) ;
+	}
+
+	return m_settings.value( "EnvironmentalVariableVolumeKey" ).toString() ;
+}
+
+void settings::removeKey( const QString& key )
+{
+	m_settings.remove( key ) ;
+}
+
 QString settings::walletName()
 {
 	return "SiriKali" ;
@@ -387,20 +407,7 @@ bool settings::unMountVolumesOnLogout()
 	return m_settings.value( "UnMountVolumesOnLogout" ).toBool() ;
 }
 
-favorites::entry settings::readFavorite( const QString& e )
-{
-	for( const auto& it : this->readFavorites() ){
-
-		if( it.volumePath == e ){
-
-			return it ;
-		}
-	}
-
-	return {} ;
-}
-
-void settings::readFavorites( QMenu * m )
+bool settings::readFavorites( QMenu * m )
 {
 	m->clear() ;
 
@@ -419,7 +426,7 @@ void settings::readFavorites( QMenu * m )
 
 	m->addSeparator() ;
 
-	const auto favorites = this->readFavorites() ;
+	const auto favorites = favorites::instance().readFavorites() ;
 
 	bool cipherPathRepeatsInFavoritesList = false ;
 
@@ -462,6 +469,8 @@ void settings::readFavorites( QMenu * m )
 			m->addAction( _add_action( e,e ) ) ;
 		}
 	}
+
+	return _showCipherPathAndMountPath ;
 }
 
 void settings::setDefaultMountPointPrefix( const QString& path )
@@ -611,6 +620,36 @@ QString settings::runCommandOnMount()
 	}
 }
 
+QString settings::runCommandOnInterval()
+{
+	if( !m_settings.contains( "runCommandOnInterval" ) ){
+
+		m_settings.setValue( "runCommandOnInterval",QString() ) ;
+	}
+
+	return m_settings.value( "runCommandOnInterval" ).toString() ;
+}
+
+void settings::runCommandOnInterval( const QString& e )
+{
+	m_settings.setValue( "runCommandOnInterval",e ) ;
+}
+
+int settings::runCommandOnIntervalTime()
+{
+	if( !m_settings.contains( "runCommandOnIntervalTime" ) ){
+
+		m_settings.setValue( "runCommandOnIntervalTime",10 ) ;
+	}
+
+	return m_settings.value( "runCommandOnIntervalTime" ).toInt() ;
+}
+
+void settings::runCommandOnIntervalTime( int e )
+{
+	m_settings.setValue( "runCommandOnIntervalTime",e ) ;
+}
+
 bool settings::reUseMountPoint()
 {
 	if( m_settings.contains( "ReUseMountPoint" ) ){
@@ -659,6 +698,21 @@ bool settings::autoCheck()
 	}
 }
 
+void settings::allowExternalToolsToReadPasswords( bool e )
+{
+	m_settings.setValue( "allowExternalToolsToReadPasswords",e ) ;
+}
+
+bool settings::allowExternalToolsToReadPasswords()
+{
+	if( !m_settings.contains( "allowExternalToolsToReadPasswords" ) ){
+
+		m_settings.setValue( "allowExternalToolsToReadPasswords",false ) ;
+	}
+
+	return m_settings.value( "allowExternalToolsToReadPasswords" ).toBool() ;
+}
+
 bool settings::getOpenVolumeReadOnlyOption()
 {
 	return readOnlyWarning::getOpenVolumeReadOnlyOption() ;
@@ -805,6 +859,21 @@ bool settings::autoMountFavoritesOnAvailable()
 	}
 }
 
+void settings::showFavoritesInContextMenu( bool e )
+{
+	m_settings.setValue( "ShowFavoritesInContextMenu",e ) ;
+}
+
+bool settings::showFavoritesInContextMenu()
+{
+	if( !m_settings.contains( "ShowFavoritesInContextMenu" ) ){
+
+		m_settings.setValue( "ShowFavoritesInContextMenu",false ) ;
+	}
+
+	return m_settings.value( "ShowFavoritesInContextMenu" ).toBool() ;
+}
+
 int settings::networkTimeOut()
 {
 	if( m_settings.contains( "NetworkTimeOut" ) ){
@@ -855,61 +924,14 @@ bool settings::ecryptfsAllowNotEncryptingFileNames()
 	return m_settings.value( "EcryptfsAllowNotEncryptingFileNames" ).toBool() ;
 }
 
-void settings::addToFavorite( const QStringList& e )
+QString settings::ykchalrespArguments()
 {
-	if( !e.isEmpty() ){
+	if( !m_settings.contains( "YkchalrespArguments" ) ){
 
-		m_settings.setValue( "FavoritesVolumes",[ & ](){
-
-			auto q = this->readFavorites() ;
-
-			q.emplace_back( e ) ;
-
-			QStringList l ;
-
-			for( const auto& it : q ){
-
-				l.append( it.configString() ) ;
-			}
-
-			return l ;
-		}() ) ;
+		m_settings.setValue( "YkchalrespArguments","-2 -i -" ) ;
 	}
-}
 
-std::vector< favorites::entry > settings::readFavorites()
-{
-	if( m_settings.contains( "FavoritesVolumes" ) ){
-
-		std::vector< favorites::entry > e ;
-
-		for( const auto& it : m_settings.value( "FavoritesVolumes" ).toStringList() ){
-
-			e.emplace_back( it ) ;
-		}
-
-		return e ;
-	}else{
-		return {} ;
-	}
-}
-
-void settings::removeFavoriteEntry( const favorites::entry& e )
-{
-	m_settings.setValue( "FavoritesVolumes",[ & ](){
-
-		QStringList l ;
-
-		for( const auto& it : this->readFavorites() ){
-
-			if( it != e ){
-
-				l.append( it.configString() ) ;
-			}
-		}
-
-		return l ;
-	}() ) ;
+	return m_settings.value( "YkchalrespArguments" ).toString() ;
 }
 
 QString settings::localizationLanguage()
