@@ -56,8 +56,8 @@
 
 #include "task.hpp"
 #include "lxqt_wallet.h"
-#include "favorites.h"
 #include "plugins.h"
+#include "secrets.h"
 #include "utility2.h"
 #include "debugwindow.h"
 
@@ -72,8 +72,27 @@ class QEvent ;
 
 namespace utility
 {
-	using volumeList = std::vector< std::pair< favorites::entry,QByteArray > > ;
+	class raii
+	{
+	public:
+		raii( std::function< void() > s ) : m_function( std::move( s ) )
+		{
+		}
+		~raii()
+		{
+			if( m_run ){
 
+				m_function() ;
+			}
+		}
+		void cancel()
+		{
+			m_run = false ;
+		}
+	private:
+		bool m_run = true ;
+		std::function< void() > m_function ;
+	};
 	template< typename T >
 	class result
 	{
@@ -235,6 +254,7 @@ namespace utility
 	bool platformIsLinux() ;
 	bool platformIsOSX() ;
 	bool platformIsWindows() ;
+	bool platformIsNOTWindows() ;
 
 	void setWindowOptions( QDialog * ) ;
 
@@ -242,6 +262,8 @@ namespace utility
 
 	bool printVersionOrHelpInfo( const QStringList& ) ;
 
+	QString getKey( const QString& keyID,const secrets& secret ) ;
+
 	wallet getKey( const QString& keyID,LXQt::Wallet::Wallet&,QWidget * = nullptr ) ;
 
 	QString cmdArgumentValue( const QStringList&,const QString& arg,const QString& defaulT = QString() ) ;
@@ -256,6 +278,7 @@ namespace utility
 
 	QString freeWindowsDriveLetter() ;
 	bool isDriveLetter( const QString& ) ;
+	bool startsWithDriveLetter( const QString& ) ;
 
 	void setWindowsMountPointOptions( QWidget *,QTextEdit *,QPushButton * ) ;
 
@@ -263,6 +286,8 @@ namespace utility
 
 	QString homeConfigPath( const QString& = QString() ) ;
 
+	QString getExistingFile( QWidget *,const QString& caption,const QString& dir ) ;
+
 	QString getExistingDirectory( QWidget *,const QString& caption,const QString& dir ) ;
 
 	bool enablePolkit( void ) ;
@@ -280,14 +305,19 @@ namespace utility
 
 	bool runningOnBackGroundThread( void ) ;
 
+	void runInUiThread( std::function< void() > function ) ;
+
 	void waitForOneSecond( void ) ;
 
 	void wait( int ) ;
 
 	void waitForFinished( QProcess& ) ;
 
-	template< typename Future >
-	static inline auto unwrap( Future& x )
+	void setMainQWidget( QWidget * ) ;
+	QWidget * mainQWidget() ;
+
+	template< typename T >
+	static inline auto unwrap( Task::future< T >& x )
 	{
 		if( utility::runningOnGUIThread() ){
 
@@ -320,9 +350,12 @@ namespace utility
 
 	void setDefaultMountPointPrefix( const QString& path ) ;
 
+	utility::result< QByteArray > yubiKey( const QString& challenge ) ;
+
 	QString mountPathPostFix( const QString& path ) ;
 	QString mountPathPostFix( const QString& prefix,const QString& path ) ;
 
+	bool pathIsFile( const QString& ) ;
 	bool pathIsReadable( const QString&,bool isFolder = true ) ;
 	bool pathIsWritable( const QString&,bool isFolder = true ) ;
 
@@ -417,20 +450,24 @@ namespace utility
 		return false ;
 	}
 
-	template< typename E,typename ... F >
-	bool containsAtleastOne( const E& e,const F& ... f )
+	template< typename E >
+	bool containsAtleastOne( const E& e )
 	{
-		for( const auto& it : { f ... } ){
-
-			if( e.contains( it ) ){
-
-				return true ;
-			}
-		}
-
+		Q_UNUSED( e )
 		return false ;
 	}
 
+	template< typename E,typename F,typename ... G >
+	bool containsAtleastOne( const E& e,F&& f,G&& ... g )
+	{
+		if( e.contains( f ) ){
+
+			return true ;
+		}else{
+			return containsAtleastOne( e,std::forward< G >( g ) ... ) ;
+		}
+	}
+
 	template< typename E,typename ... F >
 	bool startsWithAtLeastOne( const E& e,const F& ... f )
 	{
@@ -599,6 +636,10 @@ namespace utility
 		{
 			return m_finished ;
 		}
+		::Task::process::result toProcessResult()
+		{
+			return { m_stdOut,m_stdError,m_exitCode,m_exitStatus,m_finished } ;
+		}
 	private:
 		void execute( const QString& exe,int waitTime,const QProcessEnvironment& env,
 			      const QByteArray& password,std::function< void() > f,bool e ) ;
